@@ -99,7 +99,7 @@ class TextFontElementDelegate extends AnimatableStringElementDelegate {
 	}
 
 	protected void drawValue(GC gc, int x, int y, int height, boolean focused) {
-		Font font = fontList.get(value);
+		Font font = fontList.get(valueToDraw);
 		drawValue(gc, x, y, height, focused, (font != null)
 				? (font.familyName + " " + font.subfamilyName) : "      ");
 	}
@@ -219,7 +219,7 @@ class TextFontElementDelegate extends AnimatableStringElementDelegate {
 		});
 
 
-		Font currentFont = fontList.get(value);
+		Font currentFont = fontList.get(valueWithoutExpr);
 		if (currentFont != null) {
 			List<String> familyNames = fontList.listFamilyNames();
 			fontViewer.setExpandedState(currentFont.familyName, true);
