@@ -50,6 +50,8 @@ import ch.kuramo.javie.core.EffectConverter;
 import ch.kuramo.javie.core.EffectDescriptor;
 import ch.kuramo.javie.core.Interpolation;
 import ch.kuramo.javie.core.Keyframe;
+import ch.kuramo.javie.core.MediaLayer;
+import ch.kuramo.javie.core.MotionBlur;
 import ch.kuramo.javie.core.Project;
 import ch.kuramo.javie.core.ProjectDecodeException;
 import ch.kuramo.javie.core.annotations.ProjectElement;
@@ -157,6 +159,10 @@ public class ProjectDecoderImpl implements ProjectDecoder {
 							clazz = effectClass;
 						} else {
 							clazz = _elementFactory.getClass(elementType);
+
+							if (MediaLayer.class.isAssignableFrom(clazz)) {
+								convertMotionBlur(map);
+							}
 						}
 						if (clazz == null) {
 							throw new ProjectDecodeException("no such project element found: type=" + elementType);
@@ -279,6 +285,19 @@ public class ProjectDecoderImpl implements ProjectDecoder {
 					context.convert(1, list.get(1), int.class));
 		}
 
+		private void convertMotionBlur(Map<?, ?> map) {
+			@SuppressWarnings("unchecked")
+			Map<String, Object> m = (Map<String, Object>) map;
+
+			if (m.containsKey("motionBlur")) {
+				return;
+			}
+
+			Object motionBlurEnabled = m.remove("motionBlurEnabled");
+			m.put("motionBlur", Boolean.TRUE.equals(motionBlurEnabled)
+							? MotionBlur.TRANSFORM.name() : MotionBlur.NONE.name());
+		}
+
 	}
 
 }
