@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -35,7 +35,9 @@ import ch.kuramo.javie.api.AudioMode.DataType;
 import ch.kuramo.javie.core.Composition;
 import ch.kuramo.javie.core.JavieRuntimeException;
 import ch.kuramo.javie.core.Util;
+import ch.kuramo.javie.core.annotations.ProjectElement;
 
+@ProjectElement("windowsDirectShowOutput")
 public class WindowsDirectShowOutput extends PushSourceOutput {
 
 	private static final int DS_TIME_SCALE = 10000000;
@@ -176,7 +178,18 @@ public class WindowsDirectShowOutput extends PushSourceOutput {
 	}
 
 
-	static { System.loadLibrary("DirectShowOutput"); }
+	public static final boolean AVAILABLE;
+
+	static {
+		boolean available = false;
+		try {
+			System.loadLibrary("DirectShowOutput");
+			available = true;
+		} catch (Throwable t) {
+			//t.printStackTrace();
+		}
+		AVAILABLE = available;
+	}
 
 	private native static int initialize(
 			long[] nativeObjPointer, String filename,
@@ -245,8 +258,8 @@ public class WindowsDirectShowOutput extends PushSourceOutput {
 	}
 
 	public static class VideoCompressorSettings implements Cloneable {
-		public final String moniker;
-		public final String friendlyName;
+		public /*final*/ String moniker;
+		public /*final*/ String friendlyName;
 		public double quality;
 		public int keyFrameRate;
 		public int PFramesPerKey;
@@ -254,6 +267,8 @@ public class WindowsDirectShowOutput extends PushSourceOutput {
 		public long windowSize;
 		public byte[] config;
 
+		public VideoCompressorSettings() { }
+
 		public VideoCompressorSettings(VideoCompressorDescriptor desc) {
 			this.moniker = desc.moniker;
 			this.friendlyName = desc.friendlyName;
