@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010 Yoshikazu Kuramochi
+ * Copyright (c) 2010,2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -375,7 +375,7 @@ bail:
 HRESULT DirectShowOutput::AddVideoSource(
 	long width, long height, LONGLONG frameDuration, BOOL hasAlpha,
 	LPCWSTR vcName, double vcQuality, long vcKeyFrameRate, long vcPFramesPerKey,
-	LONGLONG vcBitRate, LONGLONG vcWindowSize, void* vcConfig, size_t vcConfigSize)
+	LONGLONG vcBitRate, LONGLONG vcWindowSize, void* vcConfig, long vcConfigSize)
 {
 	HRESULT hr;
 	IBindCtx* bindCtx = NULL;
@@ -439,7 +439,7 @@ HRESULT DirectShowOutput::AddVideoSource(
 
 			// MSDÑhLgɂƁu߂ĺAehCo̎ɉĈقȂBv
 			// ƂƂȂ̂Ŗ߂l̃`FbN͂ȂB
-			/*hr =*/ dialogs->SendDriverMessage(ICM_SETSTATE, (long)vcConfig, vcConfigSize);
+			hr = dialogs->SetState(vcConfig, vcConfigSize);
 		}
 	}
 
@@ -899,9 +899,29 @@ bail:
 	return hr;
 }
 
+HRESULT VideoCompressorUtil::GetFourCC(IBaseFilter* vcompFilter, DWORD* fourCC)
+{
+	HRESULT hr;
+	IPin* pin = NULL;
+	CMediaType mt;
+
+	BAIL_IF_FAILED(bail,
+		hr = VideoCompressorUtil::GetPin(vcompFilter, PINDIR_OUTPUT, &pin));
+
+	if (pin->ConnectionMediaType(&mt) == S_OK && IsEqualGUID(*mt.Type(), MEDIATYPE_Video)) {
+		*fourCC = FOURCCMap(mt.Subtype()).GetFOURCC();
+	} else {
+		hr = E_FAIL;
+	}
+
+bail:
+	RELEASE(pin);
+	return hr;
+}
+
 HRESULT VideoCompressorUtil::OpenConfigDialog(
 	long width, long height, LONGLONG frameDuration, BOOL hasAlpha,
-	LPCWSTR displayName, void** ioConfig, size_t* ioConfigSize, HWND hwnd)
+	LPCWSTR displayName, void** ioConfig, long* ioConfigSize, HWND hwnd)
 {
 	HRESULT hr;
 
@@ -910,12 +930,13 @@ HRESULT VideoCompressorUtil::OpenConfigDialog(
 	IBaseFilter* vcompFilter = NULL;
 	IGraphBuilder* dummyGraph = NULL;
 	IAMVfwCompressDialogs* dialogs = NULL;
+	HIC hic = NULL;
 
 	void* buffer = NULL;
 	int bufferSize = 0;
 
 	void* config = *ioConfig;
-	int configSize = *ioConfigSize;
+	long configSize = *ioConfigSize;
 
 	*ioConfig = NULL;
 	*ioConfigSize = 0;
@@ -935,27 +956,52 @@ HRESULT VideoCompressorUtil::OpenConfigDialog(
 		hr = VideoCompressorUtil::BuildDummyGraph(
 			width, height, frameDuration, hasAlpha, vcompFilter, &dummyGraph));
 
+	DWORD fourCC;
 	BAIL_IF_FAILED(bail,
-		hr = vcompFilter->QueryInterface(IID_IAMVfwCompressDialogs, (LPVOID*)&dialogs));
+		hr = VideoCompressorUtil::GetFourCC(vcompFilter, &fourCC));
 
-	if (config != NULL) {
+	hic = ICOpen(ICTYPE_VIDEO, fourCC, ICMODE_COMPRESS);
+	if (hic != NULL) {
+		if (config != NULL) {
+			ICSetState(hic, config, configSize);
+			// ݒ̐ۂɊւ炸s
+		}
+
+		if (ICConfigure(hic, (HWND)hwnd) != ICERR_OK) { hr = E_FAIL; goto bail; }
+
+		bufferSize = ICGetStateSize(hic);
+		if (bufferSize <= 0) { hr = E_FAIL; goto bail; }
+
+		buffer = malloc(bufferSize);
 		BAIL_IF_FAILED(bail,
-			hr = dialogs->SendDriverMessage(ICM_SETSTATE, (long)config, configSize));
+			hr = (buffer == NULL) ? E_OUTOFMEMORY : S_OK);
+
+		if (ICGetState(hic, buffer, bufferSize) != ICERR_OK) { hr = E_FAIL; goto bail; }
+
+	} else {
+		BAIL_IF_FAILED(bail,
+			hr = vcompFilter->QueryInterface(IID_IAMVfwCompressDialogs, (LPVOID*)&dialogs));
+
+		if (config != NULL && (size_t)config <= 0x7fffFFFF) {
+			hr = dialogs->SendDriverMessage(ICM_SETSTATE, (long)config, configSize);
+			// ݒ̐ۂɊւ炸s
+		}
+
+		BAIL_IF_FAILED(bail,
+			hr = dialogs->ShowDialog(VfwCompressDialog_Config, (HWND)hwnd));
+
+		BAIL_IF_FAILED(bail,
+			hr = dialogs->GetState(NULL, &bufferSize));
+		if (bufferSize <= 0) { hr = E_FAIL; goto bail; }
+
+		buffer = malloc(bufferSize);
+		BAIL_IF_FAILED(bail,
+			hr = (buffer == NULL) ? E_OUTOFMEMORY : S_OK);
+
+		BAIL_IF_FAILED(bail,
+			hr = dialogs->GetState(buffer, &bufferSize));
 	}
 
-	BAIL_IF_FAILED(bail,
-		hr = dialogs->ShowDialog(VfwCompressDialog_Config, (HWND)hwnd));
-
-	BAIL_IF_FAILED(bail,
-		hr = dialogs->GetState(NULL, &bufferSize));
-
-	buffer = malloc(bufferSize);
-	BAIL_IF_FAILED(bail,
-		hr = (buffer == NULL) ? E_OUTOFMEMORY : S_OK);
-
-	BAIL_IF_FAILED(bail,
-		hr = dialogs->GetState(buffer, &bufferSize));
-
 	*ioConfig = buffer;
 	*ioConfigSize = bufferSize;
 	buffer = NULL;
@@ -964,6 +1010,7 @@ HRESULT VideoCompressorUtil::OpenConfigDialog(
 
 bail:
 	if (buffer != NULL) free(buffer);
+	if (hic != NULL) ICClose(hic);
 	RELEASE(dialogs);
 	RELEASE(dummyGraph);
 	RELEASE(vcompFilter);
