@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010 Yoshikazu Kuramochi
+ * Copyright (c) 2010,2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -32,6 +32,7 @@ import java.util.Set;
 
 import javax.media.opengl.GLUniformData;
 
+import ch.kuramo.javie.api.IAnimatableBoolean;
 import ch.kuramo.javie.api.IAnimatableDouble;
 import ch.kuramo.javie.api.IAnimatableEnum;
 import ch.kuramo.javie.api.IAnimatableInteger;
@@ -68,9 +69,12 @@ public class ZoomBlur {
 	@Property("ZOOM_IN")
 	private IAnimatableEnum<Direction> direction;
 
-	@Property(value="20", min="0")
+	@Property(value="100", min="0")
 	private IAnimatableInteger maxSamples;
 
+	@Property("true")
+	private IAnimatableBoolean fast;
+
 
 	private final IVideoEffectContext context;
 
@@ -108,30 +112,70 @@ public class ZoomBlur {
 
 		Vec2d center = resolution.scale(context.value(this.center));
 		Direction direction = context.value(this.direction);
+		boolean fast = context.value(this.fast);
+
+		Set<IVideoBuffer> tmpBuffers = new HashSet<IVideoBuffer>();
+		tmpBuffers.add(source);
 
 		try {
-			IShaderProgram program;
-			double amountScale;
 			Set<GLUniformData> uniforms = new HashSet<GLUniformData>();
 
+			if (fast) {
+				double amountIn, amountOut;
+				switch (direction) {
+					case ZOOM_IN:
+						amountIn = amountOut = Math.min(amount/resolution.scale(200.0), 1) / maxSamples * 5;
+						break;
+
+					case ZOOM_OUT:
+						amountIn = amountOut = amount/resolution.scale(200.0) / maxSamples * 5;
+						break;
+
+					case ZOOM_BOTH:
+						amountIn = Math.min(amount/resolution.scale(400.0), 1) / maxSamples * 2.5;
+						amountOut = amount/resolution.scale(400.0) / maxSamples * 2.5;
+						break;
+
+					default:
+						throw new RuntimeException("unknown zoom direction: " + direction);
+				}
+				uniforms.add(new GLUniformData("amountIn", (float)Math.min(amountIn, 1)));
+				uniforms.add(new GLUniformData("amountOut", (float)amountOut));
+				uniforms.add(new GLUniformData("source", 0));
+				uniforms.add(new GLUniformData("center", 2, toFloatBuffer(center.x-bounds.x, center.y-bounds.y)));
+				uniforms.add(new GLUniformData("size", 2, toFloatBuffer(bounds.width, bounds.height)));
+				uniforms.add(new GLUniformData("maxSamples", (float)5));
+
+				for (int i = 0; i < 2; ++i) {
+					source.setTextureFilter(TextureFilter.LINEAR);
+					source.setTextureWrapMode(TextureWrapMode.CLAMP_TO_EDGE);
+					source = support.useShaderProgram(zoomBothProgram, uniforms, null, source);
+					tmpBuffers.add(source);
+				}
+
+				uniforms.clear();
+				maxSamples = (int) Math.ceil(maxSamples / 5.0);
+			}
+
+			IShaderProgram program;
 			switch (direction) {
 				case ZOOM_IN:
 					program = zoomInProgram;
-					amountScale = resolution.scale(200.0);
-					uniforms.add(new GLUniformData("amountIn", (float)Math.min(amount/amountScale, 1)));
+					amount /= resolution.scale(200.0);
+					uniforms.add(new GLUniformData("amountIn", (float)Math.min(amount, 1)));
 					break;
 
 				case ZOOM_OUT:
 					program = zoomOutProgram;
-					amountScale = resolution.scale(200.0);
-					uniforms.add(new GLUniformData("amountOut", (float)(amount/amountScale)));
+					amount /= resolution.scale(200.0);
+					uniforms.add(new GLUniformData("amountOut", (float)amount));
 					break;
 
 				case ZOOM_BOTH:
 					program = zoomBothProgram;
-					amountScale = resolution.scale(400.0);
-					uniforms.add(new GLUniformData("amountIn", (float)Math.min(amount/amountScale, 1)));
-					uniforms.add(new GLUniformData("amountOut", (float)(amount/amountScale)));
+					amount /= resolution.scale(400.0);
+					uniforms.add(new GLUniformData("amountIn", (float)Math.min(amount, 1)));
+					uniforms.add(new GLUniformData("amountOut", (float)amount));
 					break;
 
 				default:
@@ -149,7 +193,9 @@ public class ZoomBlur {
 			return support.useShaderProgram(program, uniforms, null, source);
 
 		} finally {
-			source.dispose();
+			for (IVideoBuffer vb : tmpBuffers) {
+				vb.dispose();
+			}
 		}
 	}
 
